==> vala-0.5.4.tar.gz: vala-0.5.4/ccode/valaccodenode.c <==
/* valaccodenode.vala
 *
 * Copyright (C) 2006-2008  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <ccode/valaccodenode.h>
#include <ccode/valaccodelinedirective.h>
#include <ccode/valaccodewriter.h>
#include <gobject/gvaluecollector.h>




struct _ValaCCodeNodePrivate {
	ValaCCodeLineDirective* _line;
};

#define VALA_CCODE_NODE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CCODE_NODE, ValaCCodeNodePrivate))
enum  {
	VALA_CCODE_NODE_DUMMY_PROPERTY
};
static void vala_ccode_node_real_write (ValaCCodeNode* self, ValaCCodeWriter* writer);
static void vala_ccode_node_real_write_declaration (ValaCCodeNode* self, ValaCCodeWriter* writer);
static void vala_ccode_node_real_write_combined (ValaCCodeNode* self, ValaCCodeWriter* writer);
static gpointer vala_ccode_node_parent_class = NULL;
static void vala_ccode_node_finalize (ValaCCodeNode* obj);



static void vala_ccode_node_real_write (ValaCCodeNode* self, ValaCCodeWriter* writer) {
	g_return_if_fail (self != NULL);
	g_critical ("Type `%s' does not implement abstract method `vala_ccode_node_write'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return;
}


/**
 * Writes this code node and all children with the specified C code
 * writer.
 *
 * @param writer a C code writer
 */
void vala_ccode_node_write (ValaCCodeNode* self, ValaCCodeWriter* writer) {
	VALA_CCODE_NODE_GET_CLASS (self)->write (self, writer);
}


/**
 * Writes declaration for this code node with the specified C code
 * writer if necessary.
 *
 * @param writer a C code writer
 */
static void vala_ccode_node_real_write_declaration (ValaCCodeNode* self, ValaCCodeWriter* writer) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (writer != NULL);
}


void vala_ccode_node_write_declaration (ValaCCodeNode* self, ValaCCodeWriter* writer) {
	VALA_CCODE_NODE_GET_CLASS (self)->write_declaration (self, writer);
}


/**
 * Writes declaration and implementation combined for this code node and
 * all children with the specified C code writer.
 *
 * @param writer a C code writer
 */
static void vala_ccode_node_real_write_combined (ValaCCodeNode* self, ValaCCodeWriter* writer) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (writer != NULL);
	vala_ccode_node_write_declaration (self, writer);
	vala_ccode_node_write (self, writer);
}


void vala_ccode_node_write_combined (ValaCCodeNode* self, ValaCCodeWriter* writer) {
	VALA_CCODE_NODE_GET_CLASS (self)->write_combined (self, writer);
}


ValaCCodeLineDirective* vala_ccode_node_get_line (ValaCCodeNode* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_line;
}


void vala_ccode_node_set_line (ValaCCodeNode* self, ValaCCodeLineDirective* value) {
	ValaCCodeLineDirective* _tmp2;
	ValaCCodeLineDirective* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_line = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : vala_ccode_node_ref (_tmp1)), (self->priv->_line == NULL) ? NULL : (self->priv->_line = (vala_ccode_node_unref (self->priv->_line), NULL)), _tmp2);
}


static void vala_value_ccode_node_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void vala_value_ccode_node_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		vala_ccode_node_unref (value->data[0].v_pointer);
	}
}


static void vala_value_ccode_node_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = vala_ccode_node_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer vala_value_ccode_node_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* vala_value_ccode_node_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		ValaCCodeNode* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = vala_ccode_node_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* vala_value_ccode_node_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	ValaCCodeNode** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = vala_ccode_node_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* vala_param_spec_ccode_node (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ValaParamSpecCCodeNode* spec;
	g_return_val_if_fail (g_type_is_a (object_type, VALA_TYPE_CCODE_NODE), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer vala_value_get_ccode_node (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALA_TYPE_CCODE_NODE), NULL);
	return value->data[0].v_pointer;
}


void vala_value_set_ccode_node (GValue* value, gpointer v_object) {
	ValaCCodeNode* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALA_TYPE_CCODE_NODE));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, VALA_TYPE_CCODE_NODE));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		vala_ccode_node_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		vala_ccode_node_unref (old);
	}
}


static void vala_ccode_node_class_init (ValaCCodeNodeClass * klass) {
	vala_ccode_node_parent_class = g_type_class_peek_parent (klass);
	VALA_CCODE_NODE_CLASS (klass)->finalize = vala_ccode_node_finalize;
	g_type_class_add_private (klass, sizeof (ValaCCodeNodePrivate));
	VALA_CCODE_NODE_CLASS (klass)->write = vala_ccode_node_real_write;
	VALA_CCODE_NODE_CLASS (klass)->write_declaration = vala_ccode_node_real_write_declaration;
	VALA_CCODE_NODE_CLASS (klass)->write_combined = vala_ccode_node_real_write_combined;
}


static void vala_ccode_node_instance_init (ValaCCodeNode * self) {
	self->priv = VALA_CCODE_NODE_GET_PRIVATE (self);
	self->ref_count = 1;
}


static void vala_ccode_node_finalize (ValaCCodeNode* obj) {
	ValaCCodeNode * self;
	self = VALA_CCODE_NODE (obj);
	(self->priv->_line == NULL) ? NULL : (self->priv->_line = (vala_ccode_node_unref (self->priv->_line), NULL));
}


GType vala_ccode_node_get_type (void) {
	static GType vala_ccode_node_type_id = 0;
	if (vala_ccode_node_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { vala_value_ccode_node_init, vala_value_ccode_node_free_value, vala_value_ccode_node_copy_value, vala_value_ccode_node_peek_pointer, "p", vala_value_ccode_node_collect_value, "p", vala_value_ccode_node_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeNodeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_node_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeNode), 0, (GInstanceInitFunc) vala_ccode_node_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		vala_ccode_node_type_id = g_type_register_fundamental (g_type_fundamental_next (), "ValaCCodeNode", &g_define_type_info, &g_define_type_fundamental_info, G_TYPE_FLAG_ABSTRACT);
	}
	return vala_ccode_node_type_id;
}


gpointer vala_ccode_node_ref (gpointer instance) {
	ValaCCodeNode* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void vala_ccode_node_unref (gpointer instance) {
	ValaCCodeNode* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		VALA_CCODE_NODE_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}

==> vala_0.7.6.orig.tar.gz: vala-0.7.6/ccode/valaccodenode.c <==
/* valaccodenode.vala
 *
 * Copyright (C) 2006-2008  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <glib.h>
#include <glib-object.h>
#include <gobject/gvaluecollector.h>


#define VALA_TYPE_CCODE_NODE (vala_ccode_node_get_type ())
#define VALA_CCODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_NODE, ValaCCodeNode))
#define VALA_CCODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_NODE, ValaCCodeNodeClass))
#define VALA_IS_CCODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_NODE))
#define VALA_IS_CCODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_NODE))
#define VALA_CCODE_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_NODE, ValaCCodeNodeClass))

typedef struct _ValaCCodeNode ValaCCodeNode;
typedef struct _ValaCCodeNodeClass ValaCCodeNodeClass;
typedef struct _ValaCCodeNodePrivate ValaCCodeNodePrivate;

#define VALA_TYPE_CCODE_WRITER (vala_ccode_writer_get_type ())
#define VALA_CCODE_WRITER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_WRITER, ValaCCodeWriter))
#define VALA_CCODE_WRITER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_WRITER, ValaCCodeWriterClass))
#define VALA_IS_CCODE_WRITER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_WRITER))
#define VALA_IS_CCODE_WRITER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_WRITER))
#define VALA_CCODE_WRITER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_WRITER, ValaCCodeWriterClass))

typedef struct _ValaCCodeWriter ValaCCodeWriter;
typedef struct _ValaCCodeWriterClass ValaCCodeWriterClass;

#define VALA_TYPE_CCODE_LINE_DIRECTIVE (vala_ccode_line_directive_get_type ())
#define VALA_CCODE_LINE_DIRECTIVE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_LINE_DIRECTIVE, ValaCCodeLineDirective))
#define VALA_CCODE_LINE_DIRECTIVE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_LINE_DIRECTIVE, ValaCCodeLineDirectiveClass))
#define VALA_IS_CCODE_LINE_DIRECTIVE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_LINE_DIRECTIVE))
#define VALA_IS_CCODE_LINE_DIRECTIVE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_LINE_DIRECTIVE))
#define VALA_CCODE_LINE_DIRECTIVE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_LINE_DIRECTIVE, ValaCCodeLineDirectiveClass))

typedef struct _ValaCCodeLineDirective ValaCCodeLineDirective;
typedef struct _ValaCCodeLineDirectiveClass ValaCCodeLineDirectiveClass;
#define _vala_ccode_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_ccode_node_unref (var), NULL)))
typedef struct _ValaParamSpecCCodeNode ValaParamSpecCCodeNode;

struct _ValaCCodeNode {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValaCCodeNodePrivate * priv;
};

struct _ValaCCodeNodeClass {
	GTypeClass parent_class;
	void (*finalize) (ValaCCodeNode *self);
	void (*write) (ValaCCodeNode* self, ValaCCodeWriter* writer);
	void (*write_declaration) (ValaCCodeNode* self, ValaCCodeWriter* writer);
	void (*write_combined) (ValaCCodeNode* self, ValaCCodeWriter* writer);
};

struct _ValaCCodeNodePrivate {
	ValaCCodeLineDirective* _line;
};

struct _ValaParamSpecCCodeNode {
	GParamSpec parent_instance;
};


static gpointer vala_ccode_node_parent_class = NULL;

gpointer vala_ccode_node_ref (gpointer instance);
void vala_ccode_node_unref (gpointer instance);
GParamSpec* vala_param_spec_ccode_node (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_ccode_node (GValue* value, gpointer v_object);
gpointer vala_value_get_ccode_node (const GValue* value);
GType vala_ccode_node_get_type (void);
gpointer vala_ccode_writer_ref (gpointer instance);
void vala_ccode_writer_unref (gpointer instance);
GParamSpec* vala_param_spec_ccode_writer (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_ccode_writer (GValue* value, gpointer v_object);
gpointer vala_value_get_ccode_writer (const GValue* value);
GType vala_ccode_writer_get_type (void);
GType vala_ccode_line_directive_get_type (void);
#define VALA_CCODE_NODE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CCODE_NODE, ValaCCodeNodePrivate))
enum  {
	VALA_CCODE_NODE_DUMMY_PROPERTY
};
void vala_ccode_node_write (ValaCCodeNode* self, ValaCCodeWriter* writer);
static void vala_ccode_node_real_write (ValaCCodeNode* self, ValaCCodeWriter* writer);
void vala_ccode_node_write_declaration (ValaCCodeNode* self, ValaCCodeWriter* writer);
static void vala_ccode_node_real_write_declaration (ValaCCodeNode* self, ValaCCodeWriter* writer);
void vala_ccode_node_write_combined (ValaCCodeNode* self, ValaCCodeWriter* writer);
static void vala_ccode_node_real_write_combined (ValaCCodeNode* self, ValaCCodeWriter* writer);
ValaCCodeNode* vala_ccode_node_construct (GType object_type);
ValaCCodeLineDirective* vala_ccode_node_get_line (ValaCCodeNode* self);
void vala_ccode_node_set_line (ValaCCodeNode* self, ValaCCodeLineDirective* value);
static void vala_ccode_node_finalize (ValaCCodeNode* obj);



static void vala_ccode_node_real_write (ValaCCodeNode* self, ValaCCodeWriter* writer) {
	g_return_if_fail (self != NULL);
	g_critical ("Type `%s' does not implement abstract method `vala_ccode_node_write'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return;
}


void vala_ccode_node_write (ValaCCodeNode* self, ValaCCodeWriter* writer) {
	VALA_CCODE_NODE_GET_CLASS (self)->write (self, writer);
}


static void vala_ccode_node_real_write_declaration (ValaCCodeNode* self, ValaCCodeWriter* writer) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (writer != NULL);
}


void vala_ccode_node_write_declaration (ValaCCodeNode* self, ValaCCodeWriter* writer) {
	VALA_CCODE_NODE_GET_CLASS (self)->write_declaration (self, writer);
}


static void vala_ccode_node_real_write_combined (ValaCCodeNode* self, ValaCCodeWriter* writer) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (writer != NULL);
	vala_ccode_node_write_declaration (self, writer);
	vala_ccode_node_write (self, writer);
}


void vala_ccode_node_write_combined (ValaCCodeNode* self, ValaCCodeWriter* writer) {
	VALA_CCODE_NODE_GET_CLASS (self)->write_combined (self, writer);
}


ValaCCodeNode* vala_ccode_node_construct (GType object_type) {
	ValaCCodeNode* self;
	self = (ValaCCodeNode*) g_type_create_instance (object_type);
	return self;
}


ValaCCodeLineDirective* vala_ccode_node_get_line (ValaCCodeNode* self) {
	ValaCCodeLineDirective* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_line;
	return result;
}


static gpointer _vala_ccode_node_ref0 (gpointer self) {
	return self ? vala_ccode_node_ref (self) : NULL;
}


void vala_ccode_node_set_line (ValaCCodeNode* self, ValaCCodeLineDirective* value) {
	ValaCCodeLineDirective* _tmp0_;
	g_return_if_fail (self != NULL);
	self->priv->_line = (_tmp0_ = _vala_ccode_node_ref0 (value), _vala_ccode_node_unref0 (self->priv->_line), _tmp0_);
}


static void vala_value_ccode_node_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void vala_value_ccode_node_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		vala_ccode_node_unref (value->data[0].v_pointer);
	}
}


static void vala_value_ccode_node_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = vala_ccode_node_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer vala_value_ccode_node_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* vala_value_ccode_node_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		ValaCCodeNode* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = vala_ccode_node_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* vala_value_ccode_node_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	ValaCCodeNode** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = vala_ccode_node_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* vala_param_spec_ccode_node (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ValaParamSpecCCodeNode* spec;
	g_return_val_if_fail (g_type_is_a (object_type, VALA_TYPE_CCODE_NODE), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer vala_value_get_ccode_node (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALA_TYPE_CCODE_NODE), NULL);
	return value->data[0].v_pointer;
}


void vala_value_set_ccode_node (GValue* value, gpointer v_object) {
	ValaCCodeNode* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALA_TYPE_CCODE_NODE));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, VALA_TYPE_CCODE_NODE));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		vala_ccode_node_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		vala_ccode_node_unref (old);
	}
}


static void vala_ccode_node_class_init (ValaCCodeNodeClass * klass) {
	vala_ccode_node_parent_class = g_type_class_peek_parent (klass);
	VALA_CCODE_NODE_CLASS (klass)->finalize = vala_ccode_node_finalize;
	g_type_class_add_private (klass, sizeof (ValaCCodeNodePrivate));
	VALA_CCODE_NODE_CLASS (klass)->write = vala_ccode_node_real_write;
	VALA_CCODE_NODE_CLASS (klass)->write_declaration = vala_ccode_node_real_write_declaration;
	VALA_CCODE_NODE_CLASS (klass)->write_combined = vala_ccode_node_real_write_combined;
}


static void vala_ccode_node_instance_init (ValaCCodeNode * self) {
	self->priv = VALA_CCODE_NODE_GET_PRIVATE (self);
	self->ref_count = 1;
}


static void vala_ccode_node_finalize (ValaCCodeNode* obj) {
	ValaCCodeNode * self;
	self = VALA_CCODE_NODE (obj);
	_vala_ccode_node_unref0 (self->priv->_line);
}


GType vala_ccode_node_get_type (void) {
	static GType vala_ccode_node_type_id = 0;
	if (vala_ccode_node_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { vala_value_ccode_node_init, vala_value_ccode_node_free_value, vala_value_ccode_node_copy_value, vala_value_ccode_node_peek_pointer, "p", vala_value_ccode_node_collect_value, "p", vala_value_ccode_node_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeNodeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_node_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeNode), 0, (GInstanceInitFunc) vala_ccode_node_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		vala_ccode_node_type_id = g_type_register_fundamental (g_type_fundamental_next (), "ValaCCodeNode", &g_define_type_info, &g_define_type_fundamental_info, G_TYPE_FLAG_ABSTRACT);
	}
	return vala_ccode_node_type_id;
}


gpointer vala_ccode_node_ref (gpointer instance) {
	ValaCCodeNode* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void vala_ccode_node_unref (gpointer instance) {
	ValaCCodeNode* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		VALA_CCODE_NODE_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}
